'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _componentEmitter = require('component-emitter');

var _componentEmitter2 = _interopRequireDefault(_componentEmitter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var emit = _componentEmitter2.default.prototype.emit;

/**
 * Use communication `KEY` to ignore other localStorage changes.
 */

/* eslint-env browser */
var KEY = '!!storage-emitter-key';

/**
 * Initialize an `Emitter` instance.
 */

var sEmitter = new _componentEmitter2.default();

/**
* Register `storage` event listener to DefaultView<window> target.
* https://developer.mozilla.org/en-US/docs/Web/Events/storage
*
* @param {StorageEvent} e { key, newValue }
*/

window.addEventListener('storage', function onStorage(e) {
  if (e.key !== KEY) return; // ignore other keys
  if (!e.newValue) return; // removeItem
  try {
    var cmd = JSON.parse(e.newValue);
    sEmitter.listeners(cmd.event).forEach(function (callback) {
      callback.call(sEmitter, cmd.args);
    });
  } catch (err) {
    console.error('unexpected value: ' + err.newValue);
  }
}, false);

/**
 * Emit `event` with the given args.
 *
 * @param {String} event
 * @param {Mixed} args
 * @return {StorageEmitter}
 */

sEmitter.emit = function (event, args) {
  var cmd = JSON.stringify({ event: event, args: args });
  localStorage.setItem(KEY, cmd);
  localStorage.removeItem(KEY);
  return emit.apply(this, arguments);
};

/**
 * Expose `sEmitter`.
 */

exports.default = sEmitter;